/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.crud;

import dk.apaq.filter.Filter;
import dk.apaq.filter.limit.Limit;
import dk.apaq.filter.sort.Sorter;
import java.util.List;

/**
 * An Interface for a Crud. The interface is split into several interfaces allowing
 * for Crud's to only implement some methods fx. for readonly cruds.
 *
 * If more functionality is required, fx. update, create, delete or filtering, then
 * the cruds need to implement the subinterfaces.
 * @param <IDT> The type used as Id for the entity, fx. String.
 * @param <BT> The entity type.
 */
public interface Crud<IDT,T> {

    /**
     * Reads a single enntity by its id.
     * @param id The id of the entity to read.
     * @return Returns the entity or null if entity available with the specified id.
     */
    T read(IDT id);

    /**
     * Lists all id's.
     * @return The list of all id's
     */
    List<IDT> listIds();
    
    /**
     * List all items. This method is potentially very resource consuing as it will 
     * return all complete objects maintained by the crud.
     * @return The list of all objects.
     */
    List<T> list();
    
    /**
     * Lists all id's.
     * @param limit Limits the returned result
     * @return The list of all id's
     */
    List<IDT> listIds(Limit limit);
    
    /**
     * List all items with a limit.
     * @return The list of all objects.
     */
    List<T> list(Limit limit);


    /**
     * Interface for editable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Editable<IDT,BT> extends Crud<IDT,BT> {
        IDT create();
        <T extends BT> IDT create(T entity);
        void update(BT entity);
        void delete(IDT id);
    }

    /**
     * Interface for filterable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Filterable<IDT,BT> extends Crud<IDT,BT> {
        List<IDT> listIds(Filter filter, Sorter sorter);
        List<IDT> listIds(Filter filter, Sorter sorter, Limit limit);
        List<BT> list(Filter filter, Sorter sorter);
        List<BT> list(Filter filter, Sorter sorter, Limit limit);
    }

    /**
     * Interface for cruds that implements all crud methods.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Complete<IDT,BT> extends Filterable<IDT,BT>, Editable<IDT, BT> { }
}
