package dk.apaq.crud.core;

import dk.apaq.filter.Filter;
import dk.apaq.filter.FiltrationItem;
import dk.apaq.filter.core.BeanFiltrationItem;
import dk.apaq.filter.limit.Limit;
import dk.apaq.filter.sort.Sorter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;
import java.util.WeakHashMap;

/**
 *
 */
public class CollectionCrud<BEANTYPE> extends BaseFilterableCrud<String, BEANTYPE> {

    private final List<BEANTYPE> beans;
    private final IdResolver<BEANTYPE> idResolver;

    private static class DefaultIdResolver<BEANTYPE> implements IdResolver<BEANTYPE> {

        private final WeakHashMap<BEANTYPE, String> idmap = new WeakHashMap<BEANTYPE, String>();

        public String getIdForBean(BEANTYPE bean) {
            String id = idmap.get(bean);
            if (id == null) {
                id = UUID.randomUUID().toString();
                idmap.put(bean, id);
            }
            return id;
        }
    }

    public interface IdResolver<BEANTYPE> {

        String getIdForBean(BEANTYPE bean);
    }

    public CollectionCrud(List<BEANTYPE> beans) {
        this(beans, new DefaultIdResolver<BEANTYPE>());
    }

    public CollectionCrud(List<BEANTYPE> beans, IdResolver<BEANTYPE> idResolver) {
        this.beans = beans;
        this.idResolver = idResolver;
    }

    public BEANTYPE read(String id) {
        fireOnBeforeRead(id);
        for (BEANTYPE current : beans) {
            String currentId = idResolver.getIdForBean(current);
            if (id.equals(currentId)) {
                fireOnRead(id, current);
                return current;
            }
        }
        return null;
    }

    public List<String> listIds(Filter filter, Sorter sorter, Limit limit) {
        List<BEANTYPE> list = new ArrayList<BEANTYPE>(beans);
        list = filter == null ? list : filterList(list, filter);
        list = sorter == null ? list : sortList(list, sorter);

        List<String> idlist = new ArrayList<String>();

        int count = 0;
        for (BEANTYPE e : list) {
            if (limit == null | (limit != null && count >= limit.getOffset() && count < (limit.getOffset() + limit.getCount()))) {
                String id = idResolver.getIdForBean(e);
                idlist.add(id);
            }
            count++;
        }
        return idlist;
    }
    
    public List<BEANTYPE> list(Filter filter, Sorter sorter, Limit limit) {
        List<String> idlist = listIds(filter, sorter, limit);
        List<BEANTYPE> beanlist = new ArrayList<BEANTYPE>();
        
        for(String id : idlist) {
            beanlist.add(read(id));
        }
        return beanlist;
        
    }


    private List<BEANTYPE> filterList(List<BEANTYPE> list, Filter filter) {
        List<BEANTYPE> filtered = new ArrayList();
        for (BEANTYPE e : list) {
            FiltrationItem fi = new BeanFiltrationItem(e);
            if (filter.passesFilter(fi)) {
                filtered.add(e);
            }
        }
        return filtered;
    }

    private List<BEANTYPE> sortList(List<BEANTYPE> list, final Sorter sorter) {
        List<BeanFiltrationItem<BEANTYPE>> sortList = new ArrayList<BeanFiltrationItem<BEANTYPE>>();
        List<BEANTYPE> sorted = new ArrayList<BEANTYPE>();

        for (BEANTYPE current : list) {
            sortList.add(new BeanFiltrationItem<BEANTYPE>(current));
        }

        Collections.sort(sortList, new Comparator<BeanFiltrationItem>() {

            @Override
            public int compare(BeanFiltrationItem o1, BeanFiltrationItem o2) {
                if (sorter == null) {
                    return 0;
                }

                switch (sorter.compare(o1, o2)) {
                    case GreaterThan:
                        return 1;
                    case LessThan:
                        return -1;
                    default:
                        return 0;
                }
            }
        });

        for (BeanFiltrationItem<BEANTYPE> current : sortList) {
            sorted.add(current.getBean());
        }
        return sorted;
    }
}
